package br.usp.ime.algorithms;

import java.util.ArrayList;
import java.util.List;

import br.usp.ime.evolvinggraph.Edge;
import br.usp.ime.evolvinggraph.EvolvingGraph;
import br.usp.ime.evolvinggraph.Interval;
import br.usp.ime.evolvinggraph.Journey;
import br.usp.ime.evolvinggraph.Node;
import br.usp.ime.journeyclass.JourneyClass;
import br.usp.ime.journeyclass.JourneyClassList;

public class FastestAlgorithm implements Algorithm {
	
	final Integer INFINITY = Integer.MAX_VALUE;

	public List<Journey> calculate(EvolvingGraph graph, Node s) {
		return calculate(graph, s, 0);
	}
	
	public JourneyClassList augmentJourneyClass(EvolvingGraph graph, Edge edge, JourneyClassList toU){
		JourneyClassList toV = new JourneyClassList();
		
		List <JourneyClass> jClasses = toU.getList();
		while (jClasses.isEmpty() == false){
			JourneyClass uJourneyClass = jClasses.remove(0);
			JourneyClass journeyClass = new JourneyClass(uJourneyClass);
			Journey journey = journeyClass.getJourney();
			int delay;
			Integer arrival = journey.getArrivalDate();
			Integer departure = edge.getNextDeparture(arrival);
			if (departure == null)
				return toV;
			Interval departureInterval = edge.getSchedule().getIntervalContaining(departure);
			Integer newDelay = departureInterval.getEnd() - departure;
			
			if (arrival + journeyClass.getDelay() <= departure){
				journey.delay(journeyClass.getDelay());
				journeyClass.setDelay(0);
				jClasses.add(journeyClass);
				continue;
			}
			if (arrival < departure){
				int splitDelay = departure - arrival;
				journey.delay(splitDelay);
				journeyClass.setDelay(journeyClass.getDelay() - splitDelay);
			}
			journeyClass.getJourney().append(edge, departure);
			delay = journeyClass.getDelay();
			if (arrival + delay > departure + newDelay){
				int splitDelay = (arrival + delay) - (departure + newDelay);
				uJourneyClass.getJourney().delay(splitDelay);
				uJourneyClass.setDelay(delay - splitDelay);
				jClasses.add(0, uJourneyClass);
				journeyClass.setDelay(splitDelay);
			}
			toV.add(journeyClass);
		}
		
		return toV;
	}

	public List<Journey> calculate(EvolvingGraph graph, Node s, int initTime) {
		int n = graph.getNumOfNodes();
		List<Journey> fastestJourneys = new ArrayList<Journey>(n);
		List<JourneyClassList> journeyClasses = new ArrayList<JourneyClassList>(n);
		List<JourneyClassList> auxJourneyClasses = new ArrayList<JourneyClassList>(n);
		
		for (int index = 0; index < n; index++){
			journeyClasses.add(index, new JourneyClassList());
			auxJourneyClasses.add(index, null);
		}
		JourneyClass jClass = new JourneyClass();
		jClass.setJourney(new Journey());
		jClass.setDelay(INFINITY);
		journeyClasses.get(s.getId()).add(jClass);
		
		for (int k = 1; k <= n; k++){
			for (int i = 0; i < n; i++)
				auxJourneyClasses.set(i, journeyClasses.get(i));
			for (Node node : graph.getNodes()){
				for (Edge edge : node.getAdj()){
					int u = edge.getU().getId();
					int v = edge.getV().getId();
					JourneyClassList lv = augmentJourneyClass(graph, edge, journeyClasses.get(u));
					auxJourneyClasses.get(v).merge(lv);
				}
			}
			for (int i = 0; i < n; i++)
				journeyClasses.set(i, auxJourneyClasses.get(i));
		}
		for (int i = 0; i < n; i++){
			fastestJourneys.add(i, journeyClasses.get(i).minimum());
		}
		
		return fastestJourneys;
	}

}
