package br.usp.ime.evolvinggraph;


import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;
import java.util.regex.MatchResult;

public class EvolvingGraph {
	List<Node> nodes;
	int edgesnum;
	
	public EvolvingGraph(int numOfNodes){
		nodes = new ArrayList<Node>();
		edgesnum = 0;
		for(int i = 0; i < numOfNodes; i++){
			nodes.add(new Node(i));
		}
	}

	public List<Node> getNodes() {
		return nodes;
	}
	
	public Node getNode(int id){
		return nodes.get(id);
	}
	
	public int getNumOfNodes() {
		return nodes.size();
	}
	
	public Edge createEdge(int u, int v, Schedule sched, int cost) {
		edgesnum++;
		Edge e = nodes.get(u).add(this.nodes.get(v), sched, cost);
		return e;
	}
	
	public Edge getEdge (int u, int v) {
		Edge e = null;
		for (Iterator<Edge> iter = this.getNode(u).getAdj().iterator(); iter.hasNext();) {
			Edge element = (Edge) iter.next();
			if (element.getV().getId() == v) {
				e = element;
				break;
			}
		}
		return e;
	}
	
	public int getNumOfEdges () {
		return this.edgesnum;
	}
	
	public String toString(){
		String str = this.getNumOfNodes() + " " + edgesnum + "\n" ;
		for (int i = 0; i < nodes.size(); i++){
			Node u = nodes.get(i);
			for (int j = 0; j < u.adj.size(); j++){
				Edge e = u.adj.get(j);
				str += u.id + " " + e.v.id + " " + e.cost + " ";
				str +=  e.schedule.size() + e.schedule.toString();
				str += "\n";
			}
		}
		return str;
	}
	
	// TODO Ler EG ja com intervalos
	public static EvolvingGraph readEvolvingGraph (String filename) {
		Schedule sched;
		int source, destiny, cost, numsched;
		int aux;
		File f = new File (filename);
		EvolvingGraph grafo = new EvolvingGraph(0);
		
		try {
			Scanner sc = new Scanner (f);
			int nodes = sc.nextInt();
			int edges = sc.nextInt();
			grafo = new EvolvingGraph(nodes);
			while(edges > 0) {
				source = sc.nextInt();
				destiny = sc.nextInt();
				cost = sc.nextInt();
				numsched = sc.nextInt();
				sched = new Schedule();
				while(numsched > 0) {
					aux = sc.nextInt();
					sched.add(aux, aux+1);
					numsched--;
				}
				grafo.createEdge(source, destiny, sched, cost);
				edges--;
			}
		}
		catch (Exception e){
			System.out.println ("Erro na leitura do grafo.");
		}
		return grafo;
	}
	
    private static void processLine(Scanner s, EvolvingGraph eg, HashSet<Edge> openConnections) {
    	Schedule sched, schedc;
        s.findInLine("(\\d+).00 CONN (\\d+) (\\d+) (\\w+)");
        MatchResult result = s.match();
        int time = Integer.parseInt((result.group(1)));
        int u = Integer.parseInt((result.group(2)));
        int v = Integer.parseInt((result.group(3)));
        String status = result.group(4);
        Edge uv = null, vu = null;
        uv = eg.getEdge(u, v);
        if (uv == null) {
        	sched = new Schedule();
        	sched.add(time, time+1);
        	schedc = new Schedule();
        	schedc.add(time, time+1);
        	Edge e1 = eg.createEdge(u, v, sched, 2);
        	Edge e2 = eg.createEdge(v, u, schedc, 2);
        	e1.setInv(e2);
        	e2.setInv(e1);
        	openConnections.add(eg.getEdge(u, v));
        	openConnections.add(eg.getEdge(u, v));
        }
        else {
        	vu = eg.getEdge(v, u);
        	sched = uv.getSchedule();
    		schedc = vu.getSchedule();
        	if (status.equals("up")) {
        		sched.add(time, time+1);
        		schedc.add(time, time+1);
        		openConnections.add(uv);
        		openConnections.add(vu);
        	}
        	else {
        		int ini = sched.last();
        		openConnections.remove(uv);
        		openConnections.remove(vu);
        		sched.add(ini, time);
        		schedc.add(ini,time);
        	}
        }
        
        //System.out.println(time + " " + u + " " + v + " " + status);
    }
    
	public static EvolvingGraph readEvolvingGraphFromONE (String filename, int numOfNodes, int simTime) {
		EvolvingGraph graph = new EvolvingGraph(numOfNodes);
		HashSet<Edge> openConnections = new HashSet<Edge> ();
		File f = new File (filename);
		Scanner sc;
		
		try {
			sc = new Scanner (f);
			while (sc.hasNextLine ()) {
				processLine(sc, graph, openConnections);
				sc.nextLine();
			}
			for (Iterator iter = openConnections.iterator(); iter.hasNext();) {
				Edge element = (Edge) iter.next();
				int ini = element.getSchedule().last();
				// Add remaining edges
				element.getSchedule().add(ini, simTime);
			}
		}
		catch (FileNotFoundException e) {
			System.out.println ("Erro na leitura do grafo.");
		}
		return graph;
	}

	public Edge getInverseEdge (Edge e) {
		return e.getInv();
	}
	
	public void removeJourney(Journey j) {
		ArrayList<Edge> edges = j.getEdges();
		ArrayList<Integer> times = j.getTimes();
		for (int i = 0; i < edges.size(); i++) {
			Edge e = edges.get(i);
			int t = times.get(i);
			
			e.getU().removeTime(e.getV(),t);
			e.getV().removeTime(e.getU(),t);
		}
	}
}
