/* 
 * Copyright 2008 TKK/ComNet
 * Released under GPLv3. See LICENSE.txt for details. 
 */
package report;

import core.DTNHost;
import core.Message;
import core.MessageListener;

/**
 * Report for of amount of messages delivered vs. time. A new report line
 * is created every time when either a message is created or delivered.
 * Messages created during the warm up period are ignored.
 * For output syntax, see {@link #HEADER}.
 */
public class MessageForemostReport extends Report implements MessageListener {
	public static String HEADER="# time  created  delivered  delivered/created";
	private Integer messageFrom;
	private Integer messageTo;
	private String messageName;

	/**
	 * Constructor.
	 */
	public MessageForemostReport() {
		init();
	}
	
	@Override
	public void init() {
		super.init();
	}

	public void newMessage(Message m) {
		if (isWarmup()) {
			addWarmupID(m.getId());
			return;
		}
		messageFrom = m.getFrom().getAddress();
		messageTo = m.getTo().getAddress();
		messageName = m.getId();
		
		reportValues();
	}
	
	/**
	 * Writes the current values to report file
	 */
	private void reportValues() {
		write(format(getSimTime()) + " " + messageName +  " " + messageFrom + " " + messageTo);
	}

	// nothing to implement for the rest
	public void messageDeleted(Message m, DTNHost where, boolean dropped) {}
	public void messageTransferAborted(Message m, DTNHost from, DTNHost to) {}
	public void messageTransferStarted(Message m, DTNHost from, DTNHost to) {}

	@Override
	public void done() {
		super.done();
	}

	public void messageTransferred(Message m, DTNHost from, DTNHost to, boolean firstDelivery) {
		return;
	}
}
